// RechenTrainer.java                               MM 2013
import Tools.IO.*;                              // Eingaben

/**
 * Rechnen, Trainer fuer eine Person.
 * Es werden zufaellig Aufgaben
 * einer Grundrechenart in einem gegebenen
 * Rechenbereich erzeugt.
 * Die eingegebenen Loesungen werden ausgewertet.
 * @author Monika Meiler
 */
public class RechenTrainer
{
/* ----------------------------------------------------- */
                                               // Attribute
/**
 * Maximale Zahl.
 */
  private int max = 20;

/**
 * Aufgabenanzahl pro Operationsblock.
 */
  private int block = 5;

/**
 * Anzahl der richtig geloesten Aufgaben.
 */
  private int richtig = 0;

/**
 * Anzahl der gestellten Aufgaben.
 */
  private int aufgaben = 0;

/* ----------------------------------------------------- */
                                             // Spielaufbau
/**
 * Spiel starten.
 */
  public void starten()
  {
    // Erlaeuterung
    System.out.println();
    erlaeuterung();

    System.out.println();
    IOTools.readLine( "Weiter (ENTER) ");

    // Spielstart
    char weiter = 'j';
    do
    {
      // Spiel
      System.out.println();
      spielen();

      // Auswertung
      System.out.println();
      auswertung();

      // Weiter
      weiter = IOTools.readChar( "Noch einmal (j/n)? ");
    } while( weiter == 'j');

    // Beenden
    System.out.println();
    System.out.println( "Spiel beendet!");
  }

/* ----------------------------------------------------- */
                                                   // Spiel
/**
 * Spielerlaeuterung.
 */
  private void erlaeuterung()
  {
    String str = "\nRechnen, ";
    str += "Trainer fuer eine Person.\n\n";
    str += "Es werden zufaellig " + block + " Aufgaben ";
    str += "einer Grundrechenart\n";
    str += "in einem gegebenen Rechenbereich erzeugt.\n";
    str += "Die eingegebenen Loesungen werden ausgewertet.";

    System.out.println( str);
  }

/**
 * Rechenbereich und Operation festlegen,
 * Aufgabenauswahl.
 */
  public void spielen()
  {
// Zahlenbereich
    System.out.println();
    eingabeMaxZahl();

// Spiel
    aufgaben = richtig = 0;
    int eingabe;
    char weiter = 'j';
    do
    {                                      // Rechenart
      System.out.println( "Auswahl der Grundrechenart");
      System.out.println( "  Addition       ... 1");
      System.out.println( "  Subtraktion    ... 2");
      System.out.println( "  Multiplikation ... 3");
      System.out.print  ( "  Division       ... 4");

      eingabe = IOTools.readInteger(": ");
      switch( eingabe)
      {
        case 1: addition(); break;
        case 2: subtraktion(); break;
        case 3: multiplikation(); break;
        case 4: division();
      }
// Weiter
      weiter = IOTools.readChar( "Noch einen Block (j/n)? ");
    } while( weiter == 'j');
  }

/**
 * Rundenauswertung.
 */
  private void auswertung()
  {
    String str = " Zahlenbereich: " + max;
    str += "\n Aufgaben: " + aufgaben;
    str += "\n Davon richtig: " + richtig;

    double prozent = richtig * 100. / aufgaben;
    str += "\n In %: " + prozent;

    int note = 5;
    if( prozent >= 40) note = 4;
    if( prozent >= 60) note = 3;
    if( prozent >= 80) note = 2;
    if( prozent >= 95) note = 1;

    switch( note)
    {
      case 1:
        str += "\n Sehr gut! Weiter so!"; break;
      case 2:
        str += "\n Gut! Weiter so!"; break;
      case 3:
        str += "\n Er ist noch kein Meister ...!"; break;
      case 4:
        str += "\n Schade! Du kannst es besser!"; break;
      case 5:
        str += "\n ben! ben! ben!";
    }

    System.out.println( str);
  }

/* ----------------------------------------------------- */
                                        // private-Methoden
/**
 * Legt groesste zulaessige Zahl fest.
 */
  private void eingabeMaxZahl()
  {
    String str = "Rechenbereich:\n";
    str += "Bis zu welcher Zahl kannst Du rechnen ";
    str += "(mindestens 20)? ";

    max = IOTools.readInteger( str);
    if( max < 20) max = 20;
  }

/* ----------------------------------------------------- */
                                             // Operationen
 /**
 * Addition.
 */
  private void addition()
  {
    int summand1, summand2, summe, ergebnis;
    aufgaben += block;

    for( int runde = 0; runde < block; runde++)
    {
// Aufgabe
      Wuerfel wuerfel1 = new Wuerfel();
      wuerfel1.setWuerfel( 0, max);
      summand1 = wuerfel1.wuerfeln();

      Wuerfel wuerfel2 = new Wuerfel();
      wuerfel2.setWuerfel( 0, max - summand1);
      summand2 = wuerfel2.wuerfeln();

      summe = summand1 + summand2;

      System.out.print( summand1 + " + " + summand2);
      ergebnis = IOTools.readInteger( " = ");

// Bewertung
      if( ergebnis == summe) richtig++;
      else
      {
        System.out.println();
        System.out.println
        ( ">>" + summand1 + " + " + summand2
                          + " = " + summe + "<<");
        System.out.println();
      }
    }
  }

/**
 * Subtraktion, als Umkehrung der Addition.
 */
  private void subtraktion()
  {
    int summand1, summand2, summe, ergebnis;
    aufgaben += block;

    for( int runde = 0; runde < block; runde++)
    {
// Aufgabe
      Wuerfel wuerfel1 = new Wuerfel();
      wuerfel1.setWuerfel( 0, max);
      summand1 = wuerfel1.wuerfeln();

      Wuerfel wuerfel2 = new Wuerfel();
      wuerfel2.setWuerfel( 0, max - summand1);
      summand2 = wuerfel2.wuerfeln();

      summe = summand1 + summand2;

      System.out.print( summe + " - " + summand1);
      ergebnis = IOTools.readInteger( " = ");

// Bewertung
      if( ergebnis == summand2) richtig++;
      else
      {
        System.out.println();
        System.out.println
        ( ">>" + summe + " - " + summand1
                       + " = " + summand2 + "<<");
        System.out.println();
      }
    }
  }

/**
 * Multiplikation.
 */
  private void multiplikation()
  {
    int faktor1, faktor2, produkt, ergebnis;
    aufgaben += block;

    for( int runde = 0; runde < block; runde++)
    {
// Aufgabe
      Wuerfel wuerfel1 = new Wuerfel();
      wuerfel1.setWuerfel( 1, max);
      faktor1 = wuerfel1.wuerfeln();

      Wuerfel wuerfel2 = new Wuerfel();
      wuerfel2.setWuerfel( 0, max / faktor1);
      faktor2 = wuerfel2.wuerfeln();

      produkt = faktor1 * faktor2;

      System.out.print( faktor1 + " * " + faktor2);
      ergebnis = IOTools.readInteger( " = ");

// Bewertung
      if( ergebnis == produkt) richtig++;
      else
      {
        System.out.println();
        System.out.println
        ( ">>" + faktor1 + " * "
               + faktor2 + " = " + produkt + "<<");
        System.out.println();
      }
    }
  }

/**
 * Division, als Umkehrung der Multiplikation.
 */
  private void division()
  {
    int faktor1, faktor2, produkt, ergebnis;
    aufgaben += block;

    for( int runde = 0; runde < block; runde++)
    {
// Aufgabe
      Wuerfel wuerfel1 = new Wuerfel();
      wuerfel1.setWuerfel( 1, max);  // >< 0
      faktor1 = wuerfel1.wuerfeln();

      Wuerfel wuerfel2 = new Wuerfel();
      wuerfel2.setWuerfel( 0, max / faktor1);
      faktor2 = wuerfel2.wuerfeln();

      produkt = faktor1 * faktor2;

      System.out.print( produkt + " : " + faktor1);
      ergebnis = IOTools.readInteger( " = ");

// Bewertung
      if( ergebnis == faktor2) richtig++;
      else
      {
        System.out.println();
        System.out.println
        ( ">>" + produkt + " : " + faktor1
                         + " = " + faktor2 + "<<");
        System.out.println();
      }
    }
  }

/* ----------------------------------------------------- */
                                                // Programm
/**
 * Startet einen Trainer.
 */
  public static void main( String[] args)
  {
// Trainer erzeugen
    RechenTrainer rechnen = new RechenTrainer();

// Traine starten
    rechnen.starten();
  }
}
